@@ -1195,6 +1195,25 @@ scan_local_path (char *mask, const char *cache_path, const char *local_path, con
     return -1;
 }
 
+static void
+extract_relative_path_from_mask (const char *mask, const char *local_path, char *trimmed_mask, char *full_local_path)
+{
+    strcpy (trimmed_mask, mask);
+    strcpy (full_local_path, local_path);
+
+    if (!mask[0] || mask[0] == '/' || mask[strlen (mask)-1] == '/')
+        return;
+
+    const char *end_of_path_in_mask = strrchr (mask, '/');
+    if (end_of_path_in_mask == NULL)
+        return;
+
+    if (full_local_path[strlen (full_local_path)-1] != '/')
+        strcat (full_local_path, "/");
+    strncat (full_local_path, mask, (size_t)(end_of_path_in_mask - mask));
+    strcpy (trimmed_mask, end_of_path_in_mask+1);
+}
+
 static int
 local_image_file (const char *cache_path, const char *local_path, const char *uri, DB_vfs_t *vfsplug)
 {
@@ -1212,12 +1231,31 @@ local_image_file (const char *cache_path, const char *local_path, const char *ur
     }
 
     for (char *mask = filemask; mask < filemask_end; mask += strlen (mask)+1) {
-        if (mask[0] && !scan_local_path (mask, cache_path, local_path, uri, vfsplug)) {
+        if (!mask[0])
+            continue;
+
+        int is_found = 0;
+        if (strrchr (mask, '/')) {
+            char *trimmed_mask = (char *)malloc (strlen (mask) + 1);
+            char *full_local_path = (char *)malloc (strlen (local_path) + strlen (mask) + 1);
+
+            extract_relative_path_from_mask (mask, local_path, trimmed_mask, full_local_path);
+            is_found = !scan_local_path (trimmed_mask, cache_path, full_local_path, uri, vfsplug);
+
+            free (trimmed_mask);
+            free (full_local_path);
+        }
+        else {
+            is_found = !scan_local_path (mask, cache_path, local_path, uri, vfsplug);
+        }
+
+        if (is_found) {
             return 0;
         }
     }
     if (!scan_local_path ("*.jpg", cache_path, local_path, uri, vfsplug) ||
-        !scan_local_path ("*.jpeg", cache_path, local_path, uri, vfsplug)) {
+        !scan_local_path ("*.jpeg", cache_path, local_path, uri, vfsplug) ||
+        !scan_local_path ("*.png", cache_path, local_path, uri, vfsplug)) {
         return 0;
     }
 
@@ -1289,7 +1327,7 @@ id3v2_artwork (const DB_id3v2_frame_t *f, int minor_version)
 //        trace ("artwork: unsupported mime type: %s\n", data);
 //        return NULL;
 //    }
-    if (*mime_end != 3) {
+    if (*mime_end != 3 && *mime_end != 0) {
         trace ("artwork: picture type=%d\n", *mime_end);
         return NULL;
     }
@@ -1699,6 +1737,29 @@ process_query (const cover_query_t *query)
         }
     }
 
+    if (artwork_enable_local && deadbeef->is_local_file (query->fname)) {
+        char *fname_copy = strdup (query->fname);
+        if (fname_copy) {
+            char *vfs_fname = vfs_path (fname_copy);
+            if (vfs_fname) {
+                /* Search inside scannable VFS containers */
+                DB_vfs_t *plugin = scandir_plug (vfs_fname);
+                if (plugin && !local_image_file (cache_path, vfs_fname, fname_copy, plugin)) {
+                    free (fname_copy);
+                    return 1;
+                }
+            }
+
+            /* Search in file directory */
+            if (!local_image_file (cache_path, dirname (vfs_fname ? vfs_fname : fname_copy), NULL, NULL)) {
+                free (fname_copy);
+                return 1;
+            }
+
+            free (fname_copy);
+        }
+    }
+
     if (artwork_enable_embedded && deadbeef->is_local_file (query->fname)) {
 #ifdef USE_METAFLAC
         // try to load embedded from flac metadata
@@ -1727,29 +1788,6 @@ process_query (const cover_query_t *query)
         }
     }
 
-    if (artwork_enable_local && deadbeef->is_local_file (query->fname)) {
-        char *fname_copy = strdup (query->fname);
-        if (fname_copy) {
-            char *vfs_fname = vfs_path (fname_copy);
-            if (vfs_fname) {
-                /* Search inside scannable VFS containers */
-                DB_vfs_t *plugin = scandir_plug (vfs_fname);
-                if (plugin && !local_image_file (cache_path, vfs_fname, fname_copy, plugin)) {
-                    free (fname_copy);
-                    return 1;
-                }
-            }
-
-            /* Search in file directory */
-            if (!local_image_file (cache_path, dirname (vfs_fname ? vfs_fname : fname_copy), NULL, NULL)) {
-                free (fname_copy);
-                return 1;
-            }
-
-            free (fname_copy);
-        }
-    }
-
 #ifdef USE_VFS_CURL
     /* Web lookups */
     if (artwork_enable_wos && strlen (query->fname) > 3 && !strcasecmp (query->fname+strlen (query->fname)-3, ".ay")) {
