@@ -57,9 +57,9 @@ typedef struct {
     WMADecodeContext wmadec;
 #endif
     int64_t first_frame_offset;
-    int currentsample;
-    int startsample;
-    int endsample;
+    int64_t currentsample;
+    int64_t startsample;
+    int64_t endsample;
     int skipsamples;
     char buffer[200000]; // can't predict its size, so set to max
     int remaining;
@@ -69,11 +69,8 @@ typedef struct {
 // allocate codec control structure
 static DB_fileinfo_t *
 wmaplug_open (uint32_t hints) {
-    DB_fileinfo_t *_info = malloc (sizeof (wmaplug_info_t));
-    wmaplug_info_t *info = (wmaplug_info_t *)_info;
-    memset (info, 0, sizeof (wmaplug_info_t));
-
-    return _info;
+    wmaplug_info_t *info = calloc (sizeof (wmaplug_info_t), 1);
+    return &info->info;
 }
 
 static DB_fileinfo_t *
@@ -83,13 +80,11 @@ wmaplug_open2 (uint32_t hints, DB_playItem_t *it) {
         return NULL;
     }
 
-    DB_fileinfo_t *_info = malloc (sizeof (wmaplug_info_t));
-    wmaplug_info_t *info = (wmaplug_info_t *)_info;
-    memset (info, 0, sizeof (wmaplug_info_t));
+    wmaplug_info_t *info = calloc (sizeof (wmaplug_info_t), 1);
     info->open2_was_used = 1;
     info->info.file = file;
 
-    return _info;
+    return &info->info;
 }
 
 int get_asf_metadata(DB_FILE *fd, DB_playItem_t *it, asf_waveformatex_t *wfx, int64_t *first_frame_offset);
@@ -152,8 +147,8 @@ wmaplug_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
         return -1;
     }
 
-    info->startsample = it->startsample;
-    info->endsample = it->endsample;
+    info->startsample = deadbeef->pl_item_get_startsample (it);
+    info->endsample = deadbeef->pl_item_get_endsample (it);
     _info->plugin = &plugin;
     _info->fmt.bps = info->wfx.bitspersample;
     _info->fmt.channels = info->wfx.channels;
@@ -163,9 +158,10 @@ wmaplug_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
     }
 
     if (!info->info.file->vfs->is_streaming ()) {
-        if (it->endsample > 0) {
-            info->startsample = it->startsample;
-            info->endsample = it->endsample;
+        int64_t endsample = deadbeef->pl_item_get_endsample (it);
+        if (endsample > 0) {
+            info->startsample = deadbeef->pl_item_get_startsample (it);
+            info->endsample = endsample;
             plugin.seek_sample (_info, 0);
         }
     }
@@ -441,32 +437,16 @@ wmaplug_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
     deadbeef->plt_set_item_duration (plt, it, totalsamples / (float)wfx.rate);
     deadbeef->pl_append_meta (it, ":FILETYPE", "WMA");
     
-    it->startsample = 0;
-    it->endsample = totalsamples-1;
+    deadbeef->pl_item_set_startsample (it, 0);
+    deadbeef->pl_item_set_endsample (it, totalsamples-1);
 
-    deadbeef->pl_lock ();
-    const char *cuesheet = deadbeef->pl_find_meta (it, "cuesheet");
-    DB_playItem_t *cue = NULL;
-
-    if (cuesheet) {
-        cue = deadbeef->plt_insert_cue_from_buffer (plt, after, it, cuesheet, strlen (cuesheet), totalsamples, wfx.rate);
-        if (cue) {
-            deadbeef->pl_item_unref (it);
-            deadbeef->pl_item_unref (cue);
-            deadbeef->pl_unlock ();
-            return cue;
-        }
-    }
-    deadbeef->pl_unlock ();
-
-    cue  = deadbeef->plt_insert_cue (plt, after, it, totalsamples, wfx.rate);
+    DB_playItem_t *cue = deadbeef->plt_process_cue (plt, after, it,  totalsamples, wfx.rate);
     if (cue) {
         deadbeef->pl_item_unref (it);
-        deadbeef->pl_item_unref (cue);
+        deadbeef->fclose (fp);
         return cue;
     }
 
-
     after = deadbeef->plt_insert_item (plt, after, it);
     deadbeef->pl_item_unref (it);
     return after;
@@ -476,8 +456,7 @@ static const char * exts[] = { "wma", NULL };
 
 // define plugin interface
 static DB_decoder_t plugin = {
-    .plugin.api_vmajor = 1,
-    .plugin.api_vminor = 7,
+    DDB_PLUGIN_SET_API_VERSION
     .plugin.version_major = 1,
     .plugin.version_minor = 0,
     .plugin.type = DB_PLUGIN_DECODER,
