@@ -37,6 +37,8 @@ settings_data_free (settings_data_t *settings_data) {
             free (settings_data->props[i].key);
         if (settings_data->props[i].title)
             free (settings_data->props[i].title);
+        if (settings_data->props[i].itemlist_type)
+            free (settings_data->props[i].itemlist_type);
     }
     memset (settings_data, 0, sizeof (settings_data_t));
 }
@@ -74,12 +76,30 @@ settings_data_init (settings_data_t *settings_data, const char *layout) {
             break;
         }
         const char *type_ptr = script;
+
         char type[MAX_TOKEN];
         script = gettoken_warn_eof (script, type);
         if (!script) {
             break;
         }
 
+        int is_hbox = !strncmp (type, "hbox[", 5);
+        int is_vbox = !strncmp (type, "vbox[", 5);
+        if (is_hbox || is_vbox) {
+            settings_data_add_property(settings_data, is_hbox ? PROP_HBOX : PROP_VBOX, NULL, NULL, NULL);
+            settings_property_t *prop = &settings_data->props[settings_data->nprops-1];
+            prop->select_options = strchr (type_ptr, '[') + 1;
+
+            char skip[MAX_TOKEN] = "";
+            do {
+                script = gettoken_warn_eof (script, skip);
+                if (!script) {
+                    break;
+                }
+            } while (strcmp (skip, ";"));
+            continue;
+        }
+
         // ignore layout options
         char key[MAX_TOKEN];
         const char *skiptokens[] = { "vert", NULL };
@@ -121,6 +141,9 @@ settings_data_init (settings_data_t *settings_data, const char *layout) {
         else if (!strcmp (type, "file")) {
             settings_data_add_property (settings_data, PROP_FILE, key, labeltext, def);
         }
+        else if (!strcmp (type, "dir")) {
+            settings_data_add_property (settings_data, PROP_DIR, key, labeltext, def);
+        }
         else if (!strcmp (type, "checkbox")) {
             settings_data_add_property (settings_data, PROP_CHECKBOX, key, labeltext, def);
         }
@@ -132,11 +155,31 @@ settings_data_init (settings_data_t *settings_data, const char *layout) {
         else if (!strncmp (type, "select[", 7)) {
             settings_data_add_property (settings_data, PROP_SELECT, key, labeltext, def);
             // skip spaces and find the first select item
-            while (*((uint8_t*)script) <= 0x20) {
+            while (*((uint8_t*)script) && *((uint8_t*)script) <= 0x20) {
                 script++;
             }
             settings_data->props[settings_data->nprops-1].select_options = script;
         }
+        else if (!strncmp (type, "itemlist<", 9)) {
+            char *p = strchr (type, '>');
+            if (!p) {
+                break;
+            }
+            settings_data_add_property (settings_data, PROP_ITEMLIST, key, labeltext, def);
+            settings_data->props[settings_data->nprops-1].itemlist_type = malloc (p-type+1);
+            memcpy (settings_data->props[settings_data->nprops-1].itemlist_type, type+9, p-type-9);
+            settings_data->props[settings_data->nprops-1].itemlist_type[p-type-9] = 0;
+        }
+        else if (!strncmp (type, "itemselect<", 11)) {
+            char *p = strchr (type, '>');
+            if (!p) {
+                break;
+            }
+            settings_data_add_property (settings_data, PROP_ITEMSELECT, key, labeltext, def);
+            settings_data->props[settings_data->nprops-1].itemlist_type = malloc (p-type+1);
+            memcpy (settings_data->props[settings_data->nprops-1].itemlist_type, type+11, p-type-11);
+            settings_data->props[settings_data->nprops-1].itemlist_type[p-type-11] = 0;
+        }
         else {
             // skip unknown/useless stuff
             char semicolon[MAX_TOKEN];
